/*
 * JBoss, Home of Professional Open Source Copyright 2009, Red Hat Middleware
 * LLC, and individual contributors by the @authors tag. See the copyright.txt
 * in the distribution for a full listing of individual contributors.
 * 
 * This is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA, or see the FSF
 * site: http://www.fsf.org.
 */
package test;

import java.io.IOException;
import java.rmi.AccessException;
import java.security.Principal;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginContext;

import org.jboss.test.security.ejb2.SimpleEJB2Session;
import org.jboss.test.security.ejb2.SimpleEJB2SessionHome;

/**
 * <p>
 * This class tests the usage of CSIv2 to propagate security information via IIOP.
 * </p>
 * 
 * @author <a href="mailto:sguilhen@redhat.com">Stefan Guilhen</a>
 */
public class CSIv2Test
{

   private Hashtable<String, Object> env;
   
   public static void main(String[] args) throws Exception
   {
      CSIv2Test test = new CSIv2Test();
      test.testCSIv2("UserA", "PassA");
      test.testCSIv2("UserB", "PassB");
      test.testCSIv2("UserC", "PassC");
   }
   
   public CSIv2Test()
   {
      // initialize the JNDI env that will be used to lookup the test EJB.
      this.env = new Hashtable<String, Object>();
      this.env.put("java.naming.factory.initial", "com.sun.jndi.cosnaming.CNCtxFactory");
      this.env.put("java.naming.provider.url", "corbaloc::localhost:3528/JBoss/Naming/root");
   }

   public void testCSIv2(String username, String password) throws Exception
   {
      // set the client security context.
      LoginContext loginContext = new LoginContext("cts", new SimpleCallbackHandler(username, password));
      loginContext.login();
      
      System.out.println("\nInvoking secure EJB2 session bean via IIOP with user " + username);
      Context context = new InitialContext(env);
      Object object = context.lookup("sample/SimpleEJB2Session");
      SimpleEJB2SessionHome home = (SimpleEJB2SessionHome) PortableRemoteObject.
         narrow(object, SimpleEJB2SessionHome.class);
      SimpleEJB2Session session = home.create();
      
      // invoke method that requires the Administrator role.
      try
      {
         Principal principal = session.invokeAdministrativeMethod();
         System.out.println("User " + principal.getName() + " successfully called administrative method!");
      }
      catch (AccessException ae)
      {
         System.out.println("User " + username + " is not authorized to call administrative method!");
      }
      
      // invoke method that requires the RegularUser role.
      try
      {
         Principal principal = session.invokeRegularMethod();
         System.out.println("User " + principal.getName() + " successfully called regular method!");
      }
      catch (AccessException ae)
      {
         System.out.println("User " + username + " is not authorized to call regular method!");
      }

      // invoke method that allows all roles.
      try
      {
         Principal principal = session.invokeUnprotectedMethod();
         System.out.println("User " + principal.getName() + " successfully called unprotected method!");
      }
      catch (AccessException ae)
      {
         // this should never happen as long as the user has successfully authenticated.
         System.out.println("User " + username + " is not authorized to call unprotected method!");
      }

      // invoke method that denies access to all roles.
      try
      {
         Principal principal = session.invokeUnavailableMethod();
         // this should never happen because the method should deny access to all roles.
         System.out.println("User " + principal.getName() + " successfully called unavailable method!");
      }
      catch (AccessException ae)
      {
         System.out.println("User " + username + " is not authorized to call unavailable method!");
      }
   }
   
   class SimpleCallbackHandler implements CallbackHandler
   {
      String username;
      char[] password;
      
      public SimpleCallbackHandler(String username, String password)
      {
         this.username = username;
         this.password = password.toCharArray();
      }
      
      @Override
      public void handle(Callback[] callbacks) throws IOException, UnsupportedCallbackException
      {
         for (Callback callback : callbacks)
         {
            if (callback instanceof NameCallback)
            {
               NameCallback nameCallback = (NameCallback) callback;
               nameCallback.setName(this.username);
            }
            else if (callback instanceof PasswordCallback)
            {
               PasswordCallback passCallback = (PasswordCallback) callback;
               passCallback.setPassword(this.password);
            }
            else
               throw new UnsupportedCallbackException(callback);
         }
      }
   }
}
